/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_BUBBLE_H__
#define __KIBA_BUBBLE_H__

#include <glib.h>
#include <cairo.h>

typedef struct _KibaBubble         KibaBubble;
typedef struct _KibaBubbleClass    KibaBubbleClass;

#include "kiba.h"
#include "kiba-object.h"

#define KIBA_TYPE_BUBBLE          (kiba_bubble_get_type ())
#define KIBA_BUBBLE(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_BUBBLE, KibaBubble))
#define KIBA_BUBBLE_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_BUBBLE,  KibaBubbleClass))
#define KIBA_IS_BUBBLE(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_BUBBLE))
#define KIBA_IS_BUBBLE_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_BUBBLE))
#define KIBA_BUBBLE_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_BUBBLE, KibaBubbleClass))

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaBubble {
	GObject    parent;
	
	PangoLayout  *layout;
	char         *message;
	char         *last_message;
};

struct _KibaBubbleClass {
	GObjectClass parent_class;
};

/**
 * Create a KibaBubble Instance.
 *
 * @return The KibaBubble Instance created.
 */
KibaBubble* kiba_bubble_new (void);

/**
 * Set message for a KibaBubble Instance.
 *
 * @param bubble The KibaBubble Instance.
 * @param message The new Message.
 * @param hide_delay The Time (in ms), the message should be displayed, or 0.
 */
void kiba_bubble_set_message (KibaBubble *bubble,
			      const char *message,
			      int         hide_delay);

/**
 * UnSet message for a KibaBubble Instance.
 *
 * @param bubble The KibaBubble Instance.
 */
void kiba_bubble_unset_message (KibaBubble *bubble);

/**
 * @param kiba The Kiba Widget.
 */
void kiba_bubble_timeout (Kiba *kiba);

/**
 * Draw the bubble Surface to a Drawing context.
 *
 * @param kiba The Kiba Widget.
 * @param cr The Drawing Context.
 * @param size The size of the Surface.
 */
void kiba_bubble_draw_surface (Kiba     *kiba,
			       cairo_t  *cr,
			       double    size);

/**
 * Draw the bubble Surface-Buffer to a Drawing context.
 *
 * @param kiba The Kiba Widget.
 * @param object The KibaObject associated to the Bubble.
 * @param cr The Drawing Context.
 */
void kiba_bubble_draw (Kiba       *kiba,
		       KibaObject *object,
		       cairo_t    *cr);

/**
 * Free allocated memory for a KibaBubble Instance.
 *
 * @param bubble The KibaSettings Instance.
 */
void kiba_bubble_fini (KibaBubble *bubble);


#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif


#endif
