/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author :	Daniel Bessler <danielb@kiba-dock.org>
 *
*/

#ifndef __KIBA_SCREEN_H__
#define __KIBA_SCREEN_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <glib-object.h>

typedef struct _KibaScreen      KibaScreen;
typedef struct _KibaScreenClass KibaScreenClass;

#include "kiba-window.h"

#define KIBA_TYPE_SCREEN          (kiba_screen_get_type ())
#define KIBA_SCREEN(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_SCREEN, KibaScreen))
#define KIBA_SCREEN_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_SCREEN,  KibaScreenClass))
#define KIBA_IS_SCREEN(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_SCREEN))
#define KIBA_IS_SCREEN_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_SCREEN))
#define KIBA_SCREEN_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_SCREEN, KibaScreenClass))

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaScreen {
	GObject     parent_instance;
	
	GdkScreen  *gdk_screen;
	GList      *windows;    /* List with all KibaWindow's managed by screen */
};

struct _KibaScreenClass {
	GObjectClass parent_class;
	
	void (* window_opened)         (KibaScreen *screen, KibaWindow *window);
	void (* window_closed)         (KibaScreen *screen, KibaWindow *window);
	void (* active_window_changed) (KibaScreen *screen, KibaWindow *active_window);
	void (* viewport_changed)      (KibaScreen *screen, GdkRectangle *new_active);
	void (* workspace_changed)     (KibaScreen *screen, gint new_active);
};

/**
 * Creates and returns a new KibaScreen object.
 *
 * @return The KibaScreen object created.
 */
KibaScreen* kiba_screen_new ();

/**
 * Runs Functions for every GdkWindow in KibaScreen's stack
 *
 * @param screen     The KibaScreen.
 * @param gdk_window The GdkWindow to search for in KibaScreen's stack.
 *
 * @return The KibaWindow associated with gdk_window, or NULL if none found.
 */
KibaWindow* kiba_screen_get_window (KibaScreen *screen,
                                    GdkWindow *gdk_window);

/**
 * Destroys and frees KibaScreen.
 *
 * @param screen The KibaScreen.
 */
void kiba_screen_fini (KibaScreen *screen);

/**
 * Runs Functions for every GdkWindow in KibaScreen's stack
 *
 * @param screen           The KibaScreen.
 * @param for_each_func (gpointer data, GdkWindow *window);
 *		@param data   The Userdata
 *		@param window The GdkWindow (Topmost Window is used first)
 * @param data The Userdata
 */
void kiba_screen_for_each_window (KibaScreen *screen,
                                  GFunc       for_each_func,
                                  gpointer    data);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif

