/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-kiba.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-kiba.org>
*/

#ifndef __KIBA_TITLE_H__
#define __KIBA_TITLE_H__

#include <gdk/gdk.h>
#include <cairo.h>

#define KIBA_TYPE_TITLE          (kiba_title_get_type ())
#define KIBA_TITLE(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_TITLE, KibaTitle))
#define KIBA_TITLE_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TITLE,  KibaTitleClass))
#define KIBA_IS_TITLE(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_TITLE))
#define KIBA_IS_TITLE_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_TITLE))
#define KIBA_TITLE_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_TITLE, KibaTitleClass))

typedef struct _KibaTitle        KibaTitle;
typedef struct _KibaTitleOptions KibaTitleOptions;
typedef struct _KibaTitleClass   KibaTitleClass;

#include "kiba-object.h"

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaTitle {
	KibaTitleOptions *options;
	GdkRectangle     geometry;           /* saves the aim geometry */
	GdkRectangle     geometry_animate;   /* saves the geometry while animating */
	double           fade_alpha;         /* stores the fade_alpha (0.0 when not shown) */
	char             *text;              /* stores the last rendered text (unallocated pointer to object->name) */
};

struct _KibaTitleOptions {
	gboolean         enable;
	gboolean         highlight_enable;
	gboolean         fade_enable;
	gboolean         disable_on_tasks;
	gchar           *rgba;
	gchar           *shadow_rgba;
	gchar           *highlight_rgba;
	gchar           *bg_rgba_1;
	gchar           *bg_rgba_2;
	gchar           *bg_rgba_3;
	gchar           *border_rgba;
	gchar           *background;
	gchar           *border_style;
	gchar           *placement_x;
	gchar           *placement_y;
	gchar           *font;
	double           shadow_offset;
	double           highlight_stepsize;
	double           fade_stop_alpha;
	double           position_stepsize;
	double           icon_padding;
	double           hide_delay;
	double           bg_radius_1;
	double           bg_radius_2;
	double           bg_step_size;
	double           bg_affection;
	double           border_width;
};

struct _KibaTitleClass {
	GObjectClass	parent_class;
};

/**
 * Creates and returns a new KibaTitle.
 *
 * @return The KibaTitle object created.
 */
KibaTitle* kiba_title_new (void);

/**
 * Draw KibaObject's title.
 *
 * @param title The KibaTitle object.
 * @param object The KibaObject.
 * @param cr The Drawing Context.
 */
void kiba_title_draw (KibaTitle  *title,
		      KibaObject *object,
		      cairo_t    *cr);

/**
 * Update KibaObject's title surface Surface Buffer.
 *
 * @param title The KibaTitle object.
 * @param object The KibaObject.
 * @param d The KibaLayoutData object.
 * @param cr The Drawing Context.
 */
void kiba_title_draw_surface (KibaTitle      *title,
			      KibaObject     *object,
			      KibaLayoutData *d,
			      cairo_t        *cr);

/**
 * 
 */
void kiba_title_timeout (Kiba       *kiba,
			 KibaTitle  *title,
			 KibaObject *object);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
