/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_H__
#define __KIBA_H__

#include "../config.h"
#ifdef HAVE_GLITZ
#ifdef CAIRO_HAS_GLITZ_SURFACE
  #define USE_GLITZ
#endif
#else
#ifdef HAVE_SDL
  #define USE_SDL
#endif
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <cairo.h>

#define KIBA_ERROR         (kiba_error_quark ())
#define KIBA_TYPE          (kiba_get_type ())
#define KIBA(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE, Kiba))
#define KIBA_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA,  KibaClass))
#define IS_KIBA(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE))
#define KIBA_IS_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE))
#define KIBA_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE, KibaClass))

/* flags macro's */
#define SET_FLAG(s,x) (s |= (1 << x))
#define DEL_FLAG(s,x) (s = s & ~(1<<x))
#define GET_FLAG(s,x) (s & (1<<x))

#define ICON_THEME      (gtk_icon_theme_get_default ())
#define FADE_STEP       (0.01)
#define SECOND_INTERVAL (1000)
#define MINUTE_INTERVAL (60000)
#define QUIT_STEPSIZE   (10)

typedef struct _Kiba           Kiba;
typedef struct _KibaClass      KibaClass;
typedef struct _KibaOptions    KibaOptions;
typedef struct _KibaLayoutData KibaLayoutData;

#include "kiba-dock.h"
#include "kiba-screen.h"
#include "kiba-title.h"
#include "kiba-object.h"
#include "kiba-render.h"

#ifdef __cplusplus
extern "C" {
#endif

enum {
	KIBA_QUIT = 0,
	KIBA_ENTERED,
	KIBA_PHYSICS,
	KIBA_DRAGGING,
	KIBA_HIDING,
	KIBA_SHAPED,
	KIBA_MOVING,
};

typedef enum {
	KIBA_POSITION_LEFT_EDGE,
	KIBA_POSITION_RIGHT_EDGE,
	KIBA_POSITION_TOP_EDGE,
	KIBA_POSITION_BOTTOM_EDGE
} KibaPosition;

struct _KibaLayoutData {
	const char      *name;
	GtkOrientation  orientation;
	gint            halignment;
	gint            valignment;
	GdkPoint        anchor;
};

static const KibaLayoutData kiba_layout_data[] = {
	{ "left_edge",    GTK_ORIENTATION_VERTICAL,   0, 1, { 0, 2 } },
	{ "right_edge",   GTK_ORIENTATION_VERTICAL,   2, 1, { 4, 2 } },
	{ "top_edge",     GTK_ORIENTATION_HORIZONTAL, 1, 0, { 2, 0 } },
	{ "bottom_edge",  GTK_ORIENTATION_HORIZONTAL, 1, 2, { 2, 4 } }
};

struct _Kiba {
	GtkWidget     widget;     /* class widget */
	int           flags;      /* state FLAGS */
	
	KibaDock      *dock;
	KibaScreen    *screen;
	KibaTitle     *title;            /* titel proberty struct */
	GList         *objects;          /* List of all KibaObject's */
	GList         *removed_objects;  /* List with removed KibaObject's */
	
	/* options */
	KibaOptions    *options;          /* settings struct */
	char           *settings_file;     /* stores the setting file content. */
	
	KibaObject     *dragging_object;     /* the currently dragged object or NULL */
	KibaObject     *prelight_object;     /* the currently prelighted object or NULL */
	KibaObject     *exec_object;         /* the current exec object or NULL */
	
	/* rendering related variables */
	GdkPixmap      *pixmap;
	GdkGC          *gc;
	KibaSurface    *progress;
	KibaSurface    *bubble;
	KibaSurface    *triangle;
	KibaSurface    *arrow;
	KibaSurface    *separator;
	gboolean        rerender;
	gboolean        shape_needs_update;
	
	/* drag animation */
	GdkRectangle        drag_geometry;
	gboolean            drag_motion;
	double              drag_window_offset;
	double              drag_alpha;
	
	/* parameter */
	gboolean    benchmark;
	int         redraws;
	int         events;
	
	/* geoemtry */
	KibaPosition         position;
	KibaLayoutData      *d;
	GdkRectangle        screen_geometry;	/* monitor/screen geometry */
	GdkRectangle        window_geometry;	/* geometry of the window */
	
	GdkRectangle        last_mouse_position;
	int                 drag_distance;       /* distance of object dragging */
	int                 dx, dy;

	int                 hide_requests;
};

struct _KibaOptions {
	gboolean         keep_window_below_others;
	gboolean         keep_window_above_others;
	gboolean         set_blur_property;
	gboolean         object_fade_enable;
	gboolean         object_fade_mask;
	gchar           *position;
	gint             monitor_number;
	double           object_animate_stepsize;
	double           object_fade_step;
	double           object_fade_alpha;
	
	/* Notification Bubble */
	gboolean         bubble_enable;
	gboolean         bubble_sliding;
	gboolean         bubble_pulse;
	gchar           *bubble_bg_rgba_1;
	gchar           *bubble_bg_rgba_2;
	gchar           *bubble_bg_rgba_3;
	gchar           *bubble_text_rgba;
	gchar           *bubble_background;
	gchar           *bubble_border_rgba;
	gchar           *bubble_border_style;
	gchar           *bubble_font;
	double           bubble_border_width;
	double           bubble_bg_affection ;
	double           bubble_bg_radius_1;
	double           bubble_bg_radius_2;
	double           bubble_slide_stepsize;
	
	/* Notification Rectangle */
	gboolean         notification_triangle_enable;
	gchar           *notification_triangle_rgba;
	
	/* progress drawing */
	gboolean         progress_enable;
	gchar           *progress_bg_rgba;
	gchar           *progress_rgba;
	
	/* AUTOHIDE */
	gboolean         auto_hide_enable;
	double           auto_hide_area;
	double           auto_hide_delay;
	double           auto_hide_speed;
	
	gchar            *mouse_over_effect;
	
	/* PULSE */
	gboolean         pulse_exec_enable;
	double           pulse_exec_stepsize;
	double           pulse_exec_zoom_factor;
	double           pulse_stepsize;
	double           pulse_max_alpha;
	double           pulse_scale_factor;
	
	/* Object switching */
	gboolean         switch_enable;
	double           switch_step;
	
	/* Object removing */
	gboolean         object_remove_flying;
	
	/* ZOOM */
	double           zoom_scale_factor;
	double           zoom_max_stepsize;
	double           zoom_area_factor;
	
	/* ROTATE */
	gboolean         mouseover_rotating;
	gboolean         exec_rotate_enable;
	gboolean         rotate_on_object_adding;
	double           mouseover_rotate_velocity;
	double           exec_rotate_velocity;
	double           rotate_scale_factor;
	
	/* BOUNCING */
	double           bounce_height;
	double           mouseover_bouncing_height;
	
	/* PRELIGHT */
	gboolean         prelight_enable;
	gchar           *prelight_rgba;
	
	/* Seperator */
	gboolean         separator_draw;
	gchar           *separator_rgba;
	gchar           *separator_inner_rgba;
	double           separator_inner_size;
};

struct _KibaClass {
	GtkWidgetClass      parent_class;
	
	void (* edge_changed) (Kiba *kiba);
	void (* class_locked) (Kiba *kiba, gchar *wm_class);
};

/**
 * Creates and returns a new Kiba Widget.
 *
 * @return The Kiba Widget created.
 */
GtkWidget* kiba_new (void);

/**
 * Hide Kiba (timeout).
 *
 * @param data The Kiba Widget.
 * @return @c TRUE if Hide is not complete, otherwise
 *         @c FALSE.
 */
gint kiba_hide (gpointer data);

/**
 * UnHide Kiba (timeout).
 *
 * @param data The Kiba Widget.
 * @return @c TRUE if UnHide is not complete, otherwise
 *         @c FALSE.
 */
gint kiba_unhide (gpointer data);

/**
 *
 */
int kiba_find_closest_position (GdkRectangle  screen_geometry,
				int           x,
				int           y);

/**
 *
 * @param kiba The Kiba Widget.
 */
void set_position_from_string (Kiba       *kiba,
			       const char *string);

/**
 *
 * @param kiba The Kiba Widget.
 */
void kiba_layout (Kiba *kiba);

/**
 *
 * @param kiba The Kiba Widget.
 */
void kiba_update_geometry (Kiba *kiba);


/**
 *
 * @param kiba The Kiba Widget.
 */
void kiba_correct_placement (Kiba *kiba);

gboolean kiba_class_is_locked (Kiba        *kiba,
			       const gchar *wm_class);

void kiba_class_lock (Kiba        *kiba,
		      const gchar *wm_class);


#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
